package CFG;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import DataFlow.DataFlow;

/**
 * Helper functions to export CFG as .vcg files.
 */
public class DumpCFG<NodeType> {
	private PrintStream out;
	private CFG<NodeType> cfg;
	private int next_node_num = 0;
	private Set<Object> visited = new HashSet<Object>();
	private Map<Object, String> nodenames = new HashMap<Object, String>();
	private DataFlow<? extends Object, NodeType> dataflow;
	
	private String escape(String string) {
		StringBuilder result = new StringBuilder();
		for (int i = 0; i < string.length(); ++i) {
			char c = string.charAt(i);
			switch (c) {
			case '"': result.append("\"");
			case '\'': result.append("\'");
			case '\\': result.append("\\\\");
			default: result.append(c);
			}
		}
		return result.toString();
	}
	
	private void dumpEdge(Object from, Object to) {
		String from_name = nodenames.get(from);
		String to_name = nodenames.get(to);
		out.printf("edge: {\n");
		out.printf("\tsourcename: \"%s\"\n", from_name);
		out.printf("\ttargetname: \"%s\"\n", to_name);
		out.printf("\tcolor: black\n");
		out.printf("}\n");
	}
	
	private void dumpNode(NodeType node) {
		if (visited.contains(node))
			return;
		visited.add(node);
		String name = "N" + (next_node_num++);
		nodenames.put(node, name);
		
		out.printf("node: {\n");
		out.printf("\ttitle: \"%s\"\n", name);
		out.printf("\tlabel: \"%s\"\n", escape(node.toString()));
		out.printf("\tcolor: lightblue\n"); /* light blue */
		/* dump dataflow results */
		if (dataflow != null) {
			out.printf("info1: \"entry: %s\n", dataflow.getEntry(node));
			out.printf("exit: %s\"\n", dataflow.getExit(node));
		}
		out.printf("}\n");
		
		for(NodeType succ : cfg.getSuccs(node)) {
			dumpNode(succ);
			dumpEdge(node, succ);
		}
		for(NodeType pred : cfg.getPreds(node)) {
			dumpNode(pred);
		}
	}
	
	public DumpCFG(PrintStream out, CFG<NodeType> cfg) {
		this.out = out;
		this.cfg = cfg;
	}
	
	/** Attaches dataflow analysis results to a dump */
	public void attachDataFlowResults(DataFlow<? extends Object, NodeType> dataflow) {
		this.dataflow = dataflow;
	}
	
	public void dump() {
		out.println("graph: {");
		out.println("layoutalgorithm: mindepth //$ \"Compilergraph\"");
		out.println("orientation: top_to_bottom");
		out.println("create_reference_nodes: no");
		out.println("create_loop_tree: no");
		out.println("layout_against_orientation: no");
		dumpNode(cfg.getStart());
		out.println("}");
	}
}
