import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.BitSet;

import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;

import AST.ASTFunction;
import CFG.ASTCFG;
import CFG.ASTCFGNode;
import CFG.DumpCFG;
import DataFlow.DataFlow;
import DataFlow.LivenessAnalysis;
import DataFlow.ReachingDefinitionAnalysis;
import Parser.SimpleLexer;
import Parser.SimpleParser;

public class Main {
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String input = args.length > 0 ? args[0] : "test.prog";
	
		try {
			FileInputStream  in = new FileInputStream(input);
			SimpleLexer lex = new SimpleLexer(new ANTLRInputStream(in));
			SimpleParser parse = new SimpleParser(new CommonTokenStream(lex));
			ASTFunction function = parse.function();
			if (parse.hadErrors) {
				System.exit(1);
				return;
			}
			
			/* build control flow graph */
			ASTCFG cfg = new ASTCFG(function);
			
			/* perform dataflow analysis */
			
			LivenessAnalysis livenessAna = new LivenessAnalysis();
			DataFlow<BitSet, ASTCFGNode> dataflow1 = new DataFlow<BitSet, ASTCFGNode>(livenessAna);
			dataflow1.perform(cfg);
			
			ReachingDefinitionAnalysis reachingAna = new ReachingDefinitionAnalysis();
			DataFlow<BitSet, ASTCFGNode> dataflow2 = new DataFlow<BitSet, ASTCFGNode>(reachingAna);
			dataflow2.perform(cfg);
			
			/* dump results */
			String output = input.replace(".prog", "") + "-liveness.vcg";
			PrintStream vcgout = new PrintStream(new FileOutputStream(output));
			DumpCFG<ASTCFGNode> dumper = new DumpCFG<ASTCFGNode>(vcgout, cfg);
			dumper.attachDataFlowResults(dataflow1);
			dumper.dump();
			vcgout.close();
			
			output = input.replace(".prog", "") + "-reaching.vcg";
			vcgout = new PrintStream(new FileOutputStream(output));
			dumper = new DumpCFG<ASTCFGNode>(vcgout, cfg);
			dumper.attachDataFlowResults(dataflow2);
			dumper.dump();
			vcgout.close();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

}
